@@ -21,6 +21,7 @@ from django.utils.html import escape
 from .forms import forms_for_survey
 from .models import (
     Answer,
+    FORMAT_CHOICES,
     OPTION_TYPE_CHOICES,
     Question,
     SURVEY_DISPLAY_TYPE_CHOICES,
@@ -31,7 +32,7 @@ from .models import (
     extra_from_filters,
     get_all_answers,
     get_filters)
-from .jsonutils import dump, dumps
+from .jsonutils import dump, dumps, datetime_to_string
 
 from .util import ChoiceEnum, get_function
 from . import settings as local_settings
@@ -63,8 +64,11 @@ def _login_url(request):
     return "/?login_required=true"
 
 
-def _get_survey_or_404(slug):
-    return get_object_or_404(Survey.live, slug=slug)
+def _get_survey_or_404(slug, request=None):
+    manager = Survey.live
+    if request and request.user.is_staff:
+        manager = Survey.objects
+    return get_object_or_404(manager, slug=slug)
 
 
 def _survey_submit(request, survey):
@@ -84,28 +88,7 @@ def _survey_submit(request, survey):
 
     forms = forms_for_survey(survey, request)
 
-    if all(form.is_valid() for form in forms):
-        submission_form = forms[0]
-        submission = submission_form.save(commit=False)
-        submission.survey = survey
-        submission.ip_address = _get_remote_ip(request)
-        submission.is_public = not survey.moderate_submissions
-        if request.user.is_authenticated():
-            submission.user = request.user
-        submission.save()
-        for form in forms[1:]:
-            answer = form.save(commit=False)
-            if isinstance(answer, (list, tuple)):
-                for a in answer:
-                    a.submission=submission
-                    a.save()
-            else:
-                if answer:
-                    answer.submission=submission
-                    answer.save()
-        # go to survey results/thanks page
-        if survey.email:
-            _send_survey_email(request, survey, submission)
+    if _submit_valid_forms(forms, request, survey):
         if survey.can_have_public_submissions():
             return _survey_results_redirect(request, survey, thanks=True)
         return _survey_show_form(request, survey, ())
@@ -113,6 +96,31 @@ def _survey_submit(request, survey):
         return _survey_show_form(request, survey, forms)
 
 
+def _submit_valid_forms(forms, request, survey):
+    if not all(form.is_valid() for form in forms):
+        return False
+    submission_form = forms[0]
+    submission = submission_form.save(commit=False)
+    submission.survey = survey
+    submission.ip_address = _get_remote_ip(request)
+    submission.is_public = not survey.moderate_submissions
+    if request.user.is_authenticated():
+        submission.user = request.user
+    submission.save()
+    for form in forms[1:]:
+        answer = form.save(commit=False)
+        if isinstance(answer, (list, tuple)):
+            for a in answer:
+                a.submission = submission
+                a.save()
+        elif answer:
+                answer.submission = submission
+                answer.save()
+    if survey.email:
+        _send_survey_email(request, survey, submission)
+    return True
+
+
 def _url_for_edit(request, obj):
     view_args = (obj._meta.app_label, obj._meta.module_name,)
     try:
@@ -164,7 +172,8 @@ def _survey_show_form(request, survey, forms):
                               dict(survey=survey,
                                    forms=forms,
                                    entered=entered,
-                                   login_url=_login_url(request)),
+                                   login_url=_login_url(request),
+                                   request=request),
                               _rc(request))
 
 
@@ -180,7 +189,7 @@ def survey_detail(request, slug):
     """ When you load the survey, this view decides what to do. It displays
     the form, redirects to the results page, displays messages, or whatever
     makes sense based on the survey, the user, and the user's entries. """
-    survey = _get_survey_or_404(slug)
+    survey = _get_survey_or_404(slug, request)
     if not survey.is_open and survey.can_have_public_submissions():
         return _survey_results_redirect(request, survey)
     need_login = (survey.is_open
@@ -199,6 +208,24 @@ def survey_detail(request, slug):
     return _survey_show_form(request, survey, forms)
 
 
+def embeded_survey_questions(request, slug):
+    survey = _get_survey_or_404(slug, request)
+    templates = ['crowdsourcing/embeded_survey_questions_%s.html' % slug,
+                 'crowdsourcing/embeded_survey_questions.html']
+    forms = ()
+    if _can_show_form(request, survey):
+        forms = forms_for_survey(survey, request)
+        if request.method == 'POST':
+            if _submit_valid_forms(forms, request, survey):
+                forms = ()
+    return render_to_response(templates, dict(
+        entered=_user_entered_survey(request, survey),
+        request=request,
+        forms=forms,
+        survey=survey,
+        login_url=_login_url(request)), _rc(request))
+
+
 def _survey_results_redirect(request, survey, thanks=False):
     response = HttpResponseRedirect(_survey_report_url(survey))
     if thanks:
@@ -212,7 +239,7 @@ def _survey_report_url(survey):
 
 
 def allowed_actions(request, slug):
-    survey = _get_survey_or_404(slug)
+    survey = _get_survey_or_404(slug, request)
     response = HttpResponse(mimetype='application/json')
     dump({"enter": _can_show_form(request, survey),
           "view": survey.can_have_public_submissions()}, response)
@@ -221,13 +248,10 @@ def allowed_actions(request, slug):
 
 def questions(request, slug):
     response = HttpResponse(mimetype='application/json')
-    dump(_get_survey_or_404(slug).to_jsondata(), response)
+    dump(_get_survey_or_404(slug, request).to_jsondata(), response)
     return response
 
 
-FORMAT_CHOICES = ('json', 'csv', 'xml', 'http',)
-
-
 def submissions(request, format):
     """ Use this view to make arbitrary queries on submissions. If the user is
     a logged in staff member, ignore submission.is_public,
@@ -263,50 +287,58 @@ def submissions(request, format):
         # we'll check it in Python, not the database.
         results = Submission.objects.filter(is_public=True)
     results = results.select_related("survey", "user")
+    get = request.GET.copy()
+    limit = int(get.pop("limit", [0])[0])
+    keys = get.keys()
     basic_filters = (
         'survey',
         'user',
         'submitted_from',
         'submitted_to',
-        'featured')
-    get = request.GET.copy()
-    keys = get.keys()
+        'featured',
+        'is_public')
     survey_slug = ""
     for field in [f for f in keys if f in basic_filters]:
         value = get[field]
+        search_field = field
         if 'survey' == field:
             search_field = 'survey__slug'
             survey_slug = value
         elif 'user' == field:
             if '' == value:
-                search_field = 'user'
                 value = None
             else:
                 search_field = 'user__username'
         elif field in ('submitted_from', 'submitted_to'):
-            format = "%Y-%m-%dT%H:%M:%S"
+            date_format = "%Y-%m-%dT%H:%M:%S"
             try:
-                value = datetime.strptime(value, format)
+                value = datetime.strptime(value, date_format)
             except ValueError:
                 return HttpResponse(
                     ("Invalid %s format. Try, for example, "
-                     "%s") % (field, datetime.now().strftime(format),))
+                     "%s") % (field, datetime.now().strftime(date_format),))
             if 'submitted_from' == field:
                 search_field = 'submitted_at__gte'
             else:
                 search_field = 'submitted_at__lte'
-        elif 'featured' == field:
+        elif field in('featured', 'is_public',):
             falses = ('f', 'false', 'no', 'n', '0',)
             value = len(value) and not value.lower() in falses
         # search_field is unicode but needs to be ascii.
         results = results.filter(**{str(search_field): value})
         get.pop(field)
+
+    def get_survey():
+        survey = Survey.objects.get(slug=survey_slug)
+        get_survey = lambda: survey
+        return survey
+
     if get:
         if survey_slug:
             results = extra_from_filters(
                 results,
                 "crowdsourcing_submission.id",
-                Survey.objects.get(slug=survey_slug),
+                get_survey(),
                 get)
         else:
             message = (
@@ -326,9 +358,15 @@ def submissions(request, format):
             message = message % (", ".join(basic_filters), item[0], item[1])
             return HttpResponse(message)
     if not is_staff:
-        rs = [r for r in results if r.survey.can_have_public_submissions()]
-        results = rs
-    answer_lookup = get_all_answers(results)
+        if survey_slug:
+            if not get_survey().can_have_public_submissions():
+                results = []
+        else:
+            rs = [r for r in results if r.survey.can_have_public_submissions()]
+            results = rs
+    if limit:
+        results = results[:limit]
+    answer_lookup = get_all_answers(results, is_staff)
     result_data = [result.to_jsondata(answer_lookup) for result in results]
 
     for data in result_data:
@@ -347,31 +385,41 @@ def submissions(request, format):
         dump(result_data, response)
     elif format == 'csv':
         response = HttpResponse(mimetype='text/csv')
+        writer = csv.writer(response)
         keys = get_keys()
         writer.writerow(keys)
         for data in result_data:
-            writer.writerow([data.get(key, "") for key in keys])
+            writer.writerow([_encode(data.get(key, "")) for key in keys])
     elif format == 'xml':
         data_list = []
         for data in result_data:
-            values = ["<%s>%s</%s>" % (k, str(v), k) for k, v in data.items()]
+            items = data.items()
+            cell = "<%s>%s</%s>"
+            values = [cell % (k, str(_encode(v)), k) for k, v in items if v]
             data_list.append("<submission>%s</submission>" % "".join(values))
         subs = "<submissions>%s</submissions>" % "\n".join(data_list)
         response = HttpResponse(subs, mimetype='text/xml')
-    elif format == 'http': # mostly for debugging.
+    elif format == 'html': # mostly for debugging.
         data_list = []
         keys = get_keys()
         results = [
             "<html><body><table>",
             "<tr>%s</tr>" % "".join(["<th>%s</th>" % k for k in keys])]
         for data in result_data:
-            cells = ["<td>%s</td>" % data.get(key, "") for key in keys]
+            cell = "<td>%s</td>"
+            cells = [cell % _encode(data.get(key, "")) for key in keys]
             results.append("<tr>%s</tr>" % "".join(cells))
         results.append("</table></body></html>")
         response = HttpResponse("\n".join(results))
+    else:
+        return HttpResponse("Unsure how to handle %s format" % format)
     return response
 
 
+def _encode(possible):
+    return datetime_to_string(possible) or possible
+    
+
 def submission(request, id):
     template = 'crowdsourcing/submission.html'
     sub = get_object_or_404(Submission.objects, is_public=True, pk=id)
@@ -435,9 +483,10 @@ def _survey_report(request, slug, report, page, templates):
             page = int(page)
         except ValueError:
             raise Http404
-    survey = _get_survey_or_404(slug)
+    survey = _get_survey_or_404(slug, request)
     # is the survey anything we can actually have a report on?
-    if not survey.can_have_public_submissions():
+    is_public = survey.is_live and survey.can_have_public_submissions()
+    if not is_public and not request.user.is_staff:
         raise Http404
     reports = survey.surveyreport_set.all()
     if report:
@@ -450,7 +499,11 @@ def _survey_report(request, slug, report, page, templates):
         report_obj = _default_report(survey)
 
     archive_fields = list(survey.get_public_archive_fields())
-    fields = list(survey.get_public_fields())
+    is_staff = request.user.is_staff
+    if is_staff:
+        fields = list(survey.get_fields())
+    else:
+        fields = list(survey.get_public_fields())
     filters = get_filters(survey, request.GET)
 
     public = survey.public_submissions()
@@ -472,7 +525,9 @@ def _survey_report(request, slug, report, page, templates):
         submissions = submissions.none()
     paginator, page_obj = paginate_or_404(submissions, page)
 
-    page_answers = get_all_answers(page_obj.object_list)
+    page_answers = get_all_answers(
+        page_obj.object_list,
+        include_private_questions=is_staff)
 
     pages_to_link = []
     for i in range(page - 5, page + 5):
@@ -495,6 +550,7 @@ def _survey_report(request, slug, report, page, templates):
         filters=filters,
         report=report_obj,
         page_answers=page_answers,
+        is_public=is_public,
         request=request)
 
     return render_to_response(templates, context, _rc(request))
